@@ -1075,29 +1075,44 @@ static int set_network_tls(struct connman_network *network,
 	const char *private_key_password;
 
 	/*
-	 * For TLS, we at least need a key, the client cert,
-	 * and a passhprase.
-	 * Server cert is optional.
+	 * For TLS, we at least need:
+	 *              The client certificate
+	 *              The client private key file
+	 *              The client private key file password
+	 *
+	 * The Authority certificate is optional.
 	 */
 	client_cert = connman_network_get_string(network,
 						"WiFi.ClientCertFile");
-	if (client_cert == NULL)
+	if (client_cert == NULL) {
+		connman_error("Error in TLS authentication: "
+			      "a ClientCertFile must be defined\n");
 		return -EINVAL;
+	}
 
 	private_key = connman_network_get_string(network,
 						"WiFi.PrivateKeyFile");
-	if (private_key == NULL)
+	if (private_key == NULL) {
+		connman_error("Error in TLS authentication: "
+			      "a PrivateKeyFile must be defined\n");
 		return -EINVAL;
+	}
 
 	private_key_password = connman_network_get_string(network,
 						"WiFi.PrivateKeyPassphrase");
-	if (private_key_password == NULL)
+	if (private_key_password == NULL) {
+		connman_error("Error in TLS authentication: "
+			      "a PrivateKeyPassphrase must be defined\n");
 		return -EINVAL;
+	}
 
 	ca_cert = connman_network_get_string(network, "WiFi.CACertFile");
 	if (ca_cert)
 		connman_dbus_dict_append_basic(dict, "ca_cert",
 						DBUS_TYPE_STRING, &ca_cert);
+	else
+		connman_info("No CACertFile has been provided "
+			     "to do the TLS authentication\n");
 
 	DBG("client cert %s private key %s", client_cert, private_key);
 
@@ -1119,19 +1134,35 @@ static int set_network_peap(struct connman_network *network,
 	char *phase2_auth;
 
 	/*
-	 * For PEAP, we at least need the sever cert, a 2nd
-	 * phase authentication and a passhprase.
-	 * Client cert is optional although strongly required
-	 * When setting the client cert, we then need a private
-	 * key as well.
+	 * For PEAP/TTLS, we at least need
+	 *              The authority certificate
+	 *              The 2nd phase authentication method
+	 *              The 2nd phase passphrase
+	 *
+	 * The Client certificate is optional although strongly required
+	 * When setting it, we need in addition
+	 *              The Client private key file
+	 *              The Client private key file password
 	 */
-	ca_cert = connman_network_get_string(network, "WiFi.CACertFile");
-	if (ca_cert == NULL)
+	if (passphrase == NULL) {
+		connman_error("Error in PEAP/TTLS authentication: "
+			      "a phase2 passphrase must be defined\n");
 		return -EINVAL;
+	}
+
+	ca_cert = connman_network_get_string(network, "WiFi.CACertFile");
+	if (ca_cert == NULL) {
+		connman_error("Error in PEAP/TTLS authentication: "
+			      "CACertFile must be defined\n");
+		return -EINVAL;
+	}
 
 	phase2 = connman_network_get_string(network, "WiFi.Phase2");
-	if (phase2 == NULL)
+	if (phase2 == NULL) {
+		connman_error("Error in PEAP/TTLS authentication: "
+			      "Phase2 must be defined\n");
 		return -EINVAL;
+	}
 
 	DBG("CA cert %s phase2 auth %s", ca_cert, phase2);
 
@@ -1142,14 +1173,22 @@ static int set_network_peap(struct connman_network *network,
 
 		private_key = connman_network_get_string(network,
 							"WiFi.PrivateKeyFile");
-		if (private_key == NULL)
+		if (private_key == NULL) {
+			connman_error("Error in PEAP/TTLS authentication: "
+				      "with ClientCertFile, "
+				      "PrivateKeyFile must be defined\n");
 			return -EINVAL;
+		}
 
 		private_key_password =
 			connman_network_get_string(network,
 						"WiFi.PrivateKeyPassphrase");
-		if (private_key_password == NULL)
+		if (private_key_password == NULL) {
+			connman_error("Error in PEAP/TTLS authentication: "
+				      "with ClientCertFile, "
+				      "PrivateKeyPassphrase must be defined\n");
 			return -EINVAL;
+		}
 
 		connman_dbus_dict_append_basic(dict, "client_cert",
 						DBUS_TYPE_STRING, &client_cert);
@@ -1162,7 +1201,9 @@ static int set_network_peap(struct connman_network *network,
 							&private_key_password);
 
 		DBG("client cert %s private key %s", client_cert, private_key);
-	}
+	} else
+		connman_info("No client certificate has been provided "
+			     "to do the PEAP/TTLS authentication\n");
 
 	phase2_auth = g_strdup_printf("\"auth=%s\"", phase2);
 
@@ -1262,7 +1303,8 @@ static int set_network(struct supplicant_task *task,
 				dbus_message_unref(message);
 				return err;
 			}
-		} else if (g_strcmp0(eap, "peap") == 0) {
+		} else if (g_strcmp0(eap, "peap") == 0 ||
+				   g_strcmp0(eap, "ttls") == 0) {
 			int err;
 
 			err = set_network_peap(network, &dict, passphrase);
@@ -1412,6 +1454,8 @@ static int initiate_scan(struct supplicant_task *task)
 	if (task->scan_call != NULL)
 		return -EALREADY;
 
+	connman_info("%s initiating scan", task->ifname);
+
 	message = dbus_message_new_method_call(SUPPLICANT_NAME, task->path,
 					SUPPLICANT_INTF ".Interface", "scan");
 	if (message == NULL)
@@ -2182,6 +2226,9 @@ static void scanning(struct supplicant_task *task, DBusMessage *msg)
 
 	connman_info("%s scanning %s", task->ifname,
 				scanning == TRUE ? "started" : "finished");
+
+	if (scanning == TRUE)
+		task->scanning = TRUE;
 }
 
 static void state_change(struct supplicant_task *task, DBusMessage *msg)
@@ -2491,6 +2538,9 @@ int supplicant_disconnect(struct connman_network *network)
 	if (task->disconnecting == TRUE)
 		return -EALREADY;
 
+	if (task->network != network)
+		return -EINVAL;
+
 	remove_network(task);
 
 	disconnect_network(task);
@@ -2500,6 +2550,25 @@ int supplicant_disconnect(struct connman_network *network)
 	return 0;
 }
 
+void supplicant_remove_network(struct connman_network *network)
+{
+	struct supplicant_task *task;
+	int index;
+
+	DBG("network %p", network);
+
+	index = connman_network_get_index(network);
+
+	task = find_task_by_index(index);
+	if (task == NULL)
+		return;
+
+	if (task->network != network)
+		return;
+
+	remove_network(task);
+}
+
 static void supplicant_activate(DBusConnection *conn)
 {
 	DBusMessage *message;
